require 'tweetstream'

module Ruboty
  module Tweets
    class Stream

      def initialize(robot)
        TweetStream.configure do |config|
          config.consumer_key       = CONSUMER_KEY
          config.consumer_secret    = CONSUMER_SECRET
          config.oauth_token        = ACCESS_TOKEN
          config.oauth_token_secret = ACCESS_TOKEN_SECRET
          config.auth_method        = :oauth
        end

        @robot  = robot
        @client = TweetStream::Client.new

        @client.on_inited do
          log(:info , "Connected to twitter stream.")

          # Every day reconnect the stream to prevent from shutdown.
          EM::PeriodicTimer.new(60 * 60 * 24) do
            @client.stream.immediate_reconnect
          end
        end

        @client.on_reconnect do |timeout, retrial|
          log(:info , "Reconnected to twitter stream: #{timeout} sec.")
        end

        @client.on_error do |message|
          log(:error, message)
        end
      end

      def start(users)
        return if users.empty?

        Thread.start do
          @client.follow(*q(users)) do |object|
            if userinfo = users[object.user.screen_name]
              message = Message.new(userinfo[:message].merge(robot: @robot))
              message.reply(u(object))
            end
          end
        end
      end

      def update(users)
        return start(users) unless @client.stream

        @client.stream.update(params: {:follow => q(users)})
      end

      def rest
        @rest ||= Twitter::REST::Client.new do |config|
          config.consumer_key        = CONSUMER_KEY
          config.consumer_secret     = CONSUMER_SECRET
          config.access_token        = ACCESS_TOKEN
          config.access_token_secret = ACCESS_TOKEN_SECRET
        end
      end

      private
      def q(users)
        users.values.map { |userinfo| userinfo[:twitter_id] }
      end

      def u(object)
        "https://twitter.com/#{object.user.screen_name}/status/#{object.id}"
      end

      def log(level, message)
        Ruboty.logger.send(level, "[#{Time.now}] ruboty-tweets: #{message}")
      end
    end
  end
end
